package com.biganto.visual.roompark.presentation.screen.settings

import com.biganto.visual.roompark.base.BaseRoomParkActivity
import com.biganto.visual.roompark.conductor.BigantoBasePresenter
import com.biganto.visual.roompark.domain.interactor.SettingsInteractor
import com.biganto.visual.roompark.domain.model.CachedDataModel
import com.biganto.visual.roompark.domain.model.SubscriptionModel
import com.biganto.visual.roompark.domain.model.TitledSubscriptionModel
import com.biganto.visual.roompark.util.monades.ExceptionString
import com.jakewharton.rxrelay2.PublishRelay
import io.reactivex.Observable
import io.reactivex.android.schedulers.AndroidSchedulers
import io.reactivex.schedulers.Schedulers
import timber.log.Timber
import java.util.concurrent.TimeUnit
import javax.inject.Inject

/**
 * Created by Vladislav Bogdashkin on 30.09.2019.
 */

private const val TYPICAL_PLAN_SIZE = (1.2f*1024L*1024L).toLong()

class SettingsScreenPresenter @Inject constructor(
    private val interactor: SettingsInteractor,
    private val activity: BaseRoomParkActivity
)
    : BigantoBasePresenter<SettingsScreen, SettingsScreenViewState>() {


    private val restoreModel = RestoreModel(mutableListOf(), arrayListOf(), arrayListOf())

    override fun detachView() {
        super.detachView()
        restoreStateObservable.accept(SettingsScreenViewState.RestoreView(restoreModel))
    }

    override fun defaultErrorViewStateHandler() =
        { e: ExceptionString -> SettingsScreenViewState.SomeError(e) }

    private val cacheSizeRefresher = PublishRelay.create<Int>()

    private val cahcObs =
        cacheSizeRefresher.flatMap {
            Observable.merge(
                arrayListOf(
                    fetchToursSize, fetchPlansSize
                )
            )
        }

    private val fetchToursSize: Observable<SettingsScreenViewState> = interactor.fetchToursSizes()
        .map { SettingsScreenViewState.OnSizePrefetch(it) }

    private val fetchPlansSize: Observable<SettingsScreenViewState>  = interactor.fetchPlanTypesSizes()
        .map { it* TYPICAL_PLAN_SIZE }
        .map { SettingsScreenViewState.OnPlanTypesPrefetch(it) }

    override fun bindIntents() {


//        val onDownloadTours = intent(SettingsScreen::downloadAllTours)
//            .flatMap { interactor.startToursDownloading()
//                .andThen(Observable.just(SettingsScreenViewState.Idle()))
//            }

        val onSubChecked = intent(SettingsScreen::onSubscription)
            .flatMap { sub ->
                interactor.switchSubscription(sub, !sub.state)
                    .map<SettingsScreenViewState> {
                        restoreModel.subs=it.toMutableList()
                        SettingsScreenViewState.SubscriptionStatus(
                            sub.id, !sub.state
                        )
                    }
                    .doOnNext {
                        val ind = restoreModel
                            .subs
                            .indexOfFirst { it.subModel.id == sub.id }
                        restoreModel.subs[ind] =
                            TitledSubscriptionModel(
                                title = restoreModel.subs[ind].title,
                                subModel = SubscriptionModel(
                                    topic = sub.topic,
                                    id = restoreModel.subs[ind].subModel.id,
                                    state = !sub.state
                                )
                            )
                    }
                    .doOnError { Timber.e(it) }
                    .onErrorReturn { SettingsScreenViewState.SubscriptionError(sub.id, sub.state) }
            }

        val fetchSettings = interactor.fetchSettings()
            .doOnNext { restoreModel.offlineStoreData = it.offlineStoreData.toMutableList() }
            .map { SettingsScreenViewState.LoadSettingsList(it) }

        val fetchCache = interactor.getCacheInfo()
            .doOnNext { cached -> cached.sortBy { it.id } }
            .doOnNext { restoreModel.cacheInfo = it }
            .map { SettingsScreenViewState.LoadCachInfo(it) }
            .doOnNext {cacheSizeRefresher.accept(1) }

        val fetchSubscriptions = interactor.getSubscriptions()
            .doOnNext { restoreModel.subs = it.toMutableList() }
            .map { SettingsScreenViewState.LoadSubscriptions(it) }

        val onSignOut = intent(SettingsScreen::signOut)
            .flatMap {
                interactor.signOut()
                    .andThen(Observable.just<SettingsScreenViewState>(SettingsScreenViewState.SignOut()))
                    .onErrorReturn(::parseError)
            }


        val refreshInfo = intent(SettingsScreen::refreshCacheInfo)
            .flatMap { fetchCache }

        val onClearCache = intent(SettingsScreen::clearCache)
            .flatMap {
                interactor.deleteCacheFiles()
                    .map<SettingsScreenViewState> {
                        Timber.d(" got progress: ${it.first} / ${it.second.toFloat()}")

                        if (it.first == it.second)
                            cacheSizeRefresher.accept(1)

                        SettingsScreenViewState.OnCacheDeleting(
                            it.first / it.second.toFloat()
                        )

                    }
                    .delay(600,TimeUnit.MILLISECONDS)
                    .startWith(SettingsScreenViewState.OnCacheDeleting(0f))
                    .doOnError { Timber.e(it) }
                    .subscribeOn(Schedulers.io())
            }

        val state = Observable.mergeDelayError(
            arrayListOf(
                restoreStateObservable,
                cahcObs,
//                fetchPlansSize,
                fetchSettings,
                onSignOut,
                onClearCache,
                refreshInfo,
                fetchSubscriptions,
                fetchCache,
                onSubChecked
//                onDownloadTours,
//                fetchToursSize
            )
        )
            .doOnError { Timber.e(it) }
            .subscribeOn(Schedulers.io())
            .onErrorReturn(::parseError)
            .observeOn(AndroidSchedulers.mainThread())

        subscribeViewState(state.cast(SettingsScreenViewState::class.java), SettingsScreen::render)
    }

}



data class RestoreModel(
    var subs: MutableList<TitledSubscriptionModel>,
    var cacheInfo:MutableList<CachedDataModel>,
    var offlineStoreData:MutableList<CachedDataModel>
)